@@ -6,24 +6,20 @@ PGObject - A toolkit integrating intelligent PostgreSQL dbs into Perl objects
 =cut
 
 package PGObject;
-
 use strict;
 use warnings;
 
-use Carp::Clan qr/^PGObject\b/;
-use Log::Any qw($log);
+use Carp;
 use Memoize;
-use PGObject::Util::DBException;
-
 use PGObject::Type::Registry;
 
 =head1 VERSION
 
-Version 2.4.0
+Version 2.0.2
 
 =cut
 
-our $VERSION = '2.4.0';
+our $VERSION = 2.000002;
 
 =head1 SYNPOSIS
 
@@ -173,9 +169,6 @@ The number of arguments
 
 =cut
 
-sub _dup_func() { '42A01' }
-sub _no_func() { '26A01' }
-
 sub function_info {
     my ( $self, %args ) = @_;
     $args{funcschema} ||= 'public';
@@ -183,7 +176,7 @@ sub function_info {
     $args{funcname} = $args{funcprefix} . $args{funcname};
     $args{argschema} ||= 'public';
 
-    my $dbh = $args{dbh} || croak $log->error( 'No dbh provided' );
+    my $dbh = $args{dbh} || croak 'No dbh provided';
 
     my $query = qq|
         SELECT proname, pronargs, proargnames,
@@ -207,36 +200,12 @@ sub function_info {
         push @queryargs, $args{argschema};
     }
 
-    my $sth = $dbh->prepare($query) || die PGObject::Util::DBException->new(
-                                            $dbh, $query, @queryargs);
-    $sth->execute(@queryargs) || die PGObject::Util::DBException->new(
-                                            $dbh, $query, @queryargs);
-    my $rows = $sth->rows;
-    if ($rows > 1) {
-        if ($args{argtype1}) {
-            my $e = PGObject::Util::DBException->internal(_dup_func,
-                    sprintf('Ambiguous criteria discovering function %s.%s (with first argument type %s)',
-                        $args{funcschema}, $args{funcname}, $args{argtype1}
-                    ), $query, @queryargs);
-            croak $e;
-        }
-        else {
-            my $e = PGObject::Util::DBException->internal(_dup_func,
-                    sprintf('Ambiguous criteria discovering function %s.%s',
-                        $args{funcschema}, $args{funcname}
-                    ), $query, @queryargs);
-            croak $e;
-        }
-    }
-    elsif ($rows == 0) {
-        my $e = PGObject::Util::DBException->internal(_no_func,
-                    sprintf('No such function: %s.%s',
-                            $args{funcschema}, $args{funcname} ),
-                    $query, @queryargs);
-
-        croak $e;
-    }
+    my $sth = $dbh->prepare($query) || die $!;
+    $sth->execute(@queryargs) || die $dbh->errstr . ": " . $query;
     my $ref = $sth->fetchrow_hashref('NAME_lc');
+    croak "transaction already aborted"  if $dbh->state eq '25P02';
+    croak "No such function"             if !$ref;
+    croak 'Ambiguous discovery criteria' if $sth->fetchrow_hashref('NAME_lc');
 
     my $f_args;
     for my $n ( @{ $ref->{proargnames} } ) {
@@ -311,10 +280,8 @@ sub call_procedure {
     $args{registry} ||= 'default';
 
     my $dbh = $args{dbh};
-    croak $log->error( "No database handle provided" )
-        unless $dbh;
-    croak $log->error( "dbh not a database handle" )
-        unless eval { $dbh->isa('DBI::db') };
+    croak "No database handle provided" unless $dbh;
+    croak "dbh not a database handle" unless eval { $dbh->isa('DBI::db') };
 
     my $wf_string = '';
 
@@ -363,8 +330,7 @@ sub call_procedure {
          ORDER BY $order |;
     }
 
-    my $sth = $dbh->prepare($query) || die PGObject::Util::DBException(
-                                        $dbh, $query, @qargs);
+    my $sth = $dbh->prepare($query) || die $!;
 
     my $place = 1;
 
@@ -391,19 +357,25 @@ sub call_procedure {
         ++$place;
     }
 
-    $sth->execute() || die PGObject::Util::DBException->new($dbh, $query, @qargs);
+    $sth->execute() || die $dbh->errstr . ": " . $query;
 
     clear_info_cache() if $dbh->state eq '42883';    # (No Such Function)
 
     my @rows = ();
-    my $row_deserializer =
-        PGObject::Type::Registry->rowhash_deserializer(
-            registry => $args{registry},
-            types    => $sth->{pg_type},
-            columns  => $sth->{NAME_lc},
-        );
-    while (my $row = $sth->fetchrow_hashref('NAME_lc')) {
-        push @rows, $row_deserializer->( $row );
+    while ( my $row = $sth->fetchrow_hashref('NAME_lc') ) {
+        my @types = @{ $sth->{pg_type} };
+        my @names = @{ $sth->{NAME_lc} };
+        my $i     = 0;
+        for my $type (@types) {
+            $row->{ $names[$i] } = PGObject::Type::Registry->deserialize(
+                registry => $args{registry},
+                dbtype   => $type,
+                dbstring => $row->{ $names[$i] }
+            );
+            ++$i;
+        }
+
+        push @rows, $row;
     }
     return @rows;
 }
@@ -423,7 +395,7 @@ This no longer returns anything of significance.
 
 sub new_registry {
     my ( $self, $registry_name ) = @_;
-    carp $log->warn( "Deprecated use of PGObject->new_registry()" );
+    carp "Deprecated use of PGObject->new_registry()";
     PGObject::Type::Registry->new_registry($registry_name);
 }
 
@@ -446,7 +418,7 @@ Use PGObject::Type::Registry->register_type() instead.
 =cut
 
 sub register_type {
-    carp $log->warn( 'Use of deprecated method register_type of PGObject module' );
+    carp 'Use of deprecated method register_type of PGObject module';
     my ( $self, %args ) = @_;
 
     PGObject::Type::Registry->register_type(
@@ -469,7 +441,7 @@ instead.
 =cut
 
 sub unregister_type {
-    carp $log->warn( 'Use of deprecated method unregister_type of PGObject' );
+    carp 'Use of deprecated method unregister_type of PGObject';
     my ( $self, %args ) = @_;
 
     $args{registry} ||= 'default';
@@ -637,9 +609,17 @@ You can also look for information at:
 
 L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=PGObject>
 
-=item * MetaCPAN
+=item * AnnoCPAN: Annotated CPAN documentation
 
-L<https://metacpan.org/dist/PGObject>
+L<http://annocpan.org/dist/PGObject>
+
+=item * CPAN Ratings
+
+L<http://cpanratings.perl.org/d/PGObject>
+
+=item * Search CPAN
+
+L<http://search.cpan.org/dist/PGObject/>
 
 =back
 
@@ -666,8 +646,8 @@ not be half of what it is today.
 
 =head1 COPYRIGHT
 
-  COPYRIGHT (C) 2013-2014 Chris Travers
-  COPYRIGHT (C) 2014-2021 The LedgerSMB Core Team
+COPYRIGHT (C) 2013-2014 Chris Travers
+COPYRIGHT (C) 2014-2017 The LedgerSMB Core Team
 
 Redistribution and use in source and compiled forms with or without
 modification, are permitted provided that the following conditions are met:
@@ -689,8 +669,6 @@ distribution.
 
 =back
 
-=head1 LICENSE
-
 THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
